#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/9/10 22:54
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : Help.py
# @Software : PyCharm



def del_redis_keys_by_prefix(prefix, conn):
    for key in conn.keys():
        if str(key).startswith(prefix):
            conn.delete(key)
            print(key)


if __name__ == '__main__':
    d = {'url': 'https://www.baidu.com/s?ie=utf8&oe=utf8&wd=%E7%9B%91%E6%8E%A7%E5%B9%B3%E5%8F%B0&tn=98010089_dg&ch=2', 'crawl_ts': 1557479433, 'create_ts': '2019-05-10 17:10:33',
         'real_url': 'http://www.tjlsty.com.cn/Home/Product/index/pid/3.html#cur_pro',
         'snapshot_url': 'http://www.baidu.com/baidu.php?url=af0000aF4eAvq9HI8keayRouCCa3UvfSC4zodAur9WJD614Ui34sRpxjQjxy9ho1ZPmq7XRjQUGzcBbr02fzKbd8LXc1erKSs-uUxPUdk2D2EnY2ujHbOo4nEBUcjmoBRxYSRQ_Khmjcu8ZR_Zt-38ZyFrOvvX0CEtLzN67PBwXBhe2eLEkNwYEAhWYbeBKf-ipcQ9JgjAYoSJAa4s.Db_a9-CkYP7nZeeV2m5wKV6k3Cr55OCf3F8gYoDkEvyNvyUPOQIJyAp7BEW342d0.U1YY0ZDqEraOvnpq1260mywkXHvt3QOmkoLnJ0K9uZ7Y5Hc0TA-W5H00Ijvt3QOm1egP_sKGUHYznjf0u1dEugK1nfKdpHdBmy-bIykV0ZKGujY10APGujYYnH60UgfqnH0krNtknjDLg1nknWwxn1msnfKopHYs0ZFY5HcsP0K-pyfqnHfYn-tznH03r7tzrjRkr7tzrjTLPNtzrjTzn7tzrjbvP7tzrj61PdtzrjR3PfKBpHYznjwxnHRd0AdW5HDsnj7xrjczPjfsnjnsg1Dsn-ts0Z7spyfqn0Kkmv-b5H00ThIYmyTqn0K9mWYsg100ugFM5H00TZ0qnWbYn101PH030A4vTjYsQW0snj0snj0s0AdYTjYs0AwbUL0qn0KzpWYs0Aw-IWdsmsKhIjYs0ZKC5H00ULnqn0KBI1Yz0A4Y5H00TLCq0A71gv-bm1dsTzdWUfKYIgnqnW64nHcvn1n4PHfYnjb4nWmYPsKzug7Y5HDdPHTYP1T1nWbsPWn0Tv-b5H9WuyP-rAcvnj0snhDsuhf0mLPV5Rn3PWFDPHcvPRf3wRDdfHn0mynqnfKsUWYs0Z7VIjYs0Z7VT1Ys0ZGY5H00UyPxuMFEUHYsg1Kxn7ts0Aw9UMNBuNqsUA78pyw15HKxn7tzP1fvrjf4g100TA7Ygvu_myTqn0Kbmv-b5H00ugwGujYVnfK9TLKWm1Ys0ZNspy4Wm1Ys0Z7VuWYkP6KhmLNY5H00uMGC5H00uh7Y5H00XMK_Ignqn0K9uAu_myTqnfK_uhnqn0KEIjYs0AqzTZfqnanscznsc10WnansQW0snj0sn0KWThnqnHmLPWT&word=%E7%9B%91%E6%8E%A7%E5%B9%B3%E5%8F%B0',
         'title': '天津联盛天远监控系统_全程一体化服务 '}
    d = {
        "albumname": "周杰伦的床边故事",
        "publishtime": "2016-06-24 00:00:00",
        "singername": "周杰伦",
        "intro": "夜深了，猫头鹰出没，数完第一千零一个尖叫声，开始听，周杰伦 的床边故事，Jay Chou’s Bedtime Stories，今年最令人激动的乐坛震撼弹，周杰伦2016年最新着魔专辑强势暗袭你的床边！超乎想象、最自由的古典灵魂摇滚嘻哈，听周杰伦说书！10个出神入化的音乐故事。",
        "songcount": 0,
        "imgurl": "http:\/\/imge.kugou.com\/stdmusic\/{size}\/20160623\/20160623233610830051.jpg",
        "albumid": 1645030,
        "collectcount": 0,
        "singerid": 3520,
        "sextype": 0,
        "privilege": 10,
        "trans_param": {
            "special_tag": "0"
        }
    }



# -*- coding: utf-8 -*-
import re
import traceback
import urllib.parse


def urlParse(k):
    return urllib.parse.unquote(str(k))


def urlEncode(k):
    return urllib.parse.quote(str(k))


def parseDict(item):
    # if isinstance(item, Item):
    #     item = parseDict(item.__dict__)
    for k in item:
        if isinstance(item[k], str):
            item[k] = urlParse(item[k])
        elif isinstance(item[k], dict):
            item[k] = parseDict(item[k])
    return item


# 从url中获取参数列表
def formatUrlToDict(url):
    res = {}
    try:
        values = url.split('?')[-1]
        for key_value in values.split('&'):
            try:
                res[key_value.split('=')[0]] = key_value.split('=')[1]
            except:
                pass
        return res
    except:
        traceback.print_exc()
        print(" === URL: " + str(url))
        return {}


def delHtml(str1):
    str1 = str1.replace("<br />", "\n").replace("<br/>", "\n").replace("<br>", "\n").replace("<hr />", "\n")
    return re.compile(r'<[^>]+>', re.S).sub('', str(str1)) if str1 else ""


def decodeUrl(url):
    import urllib.parse
    return urllib.parse.unquote(url)


# 删除字符串
def delStr(from_str, start_str, end_str):
    s_index = from_str.find(start_str)
    if s_index > 0:
        s_index = len(start_str) + s_index
        e_index = from_str.find(end_str, s_index)
        if s_index > 0 and e_index > s_index:
            from_str1 = from_str[:s_index - 1]
            from_str2 = from_str[e_index + 1:]
            from_str = from_str1 + from_str2
            return cutStr(from_str, start_str, end_str)
        else:
            return from_str
    else:
        return from_str


# 切割字符串
def cutStr(from_str, start_str, end_str):
    s_index = from_str.find(start_str)
    if s_index > 0:
        s_index = len(start_str) + s_index
        e_index = from_str.find(end_str, s_index)
        if e_index >= s_index:
            return from_str[s_index:e_index]
        else:
            return from_str[s_index:]
    return ""


def getCookiesFromList(l):
    # l = ['BDORZ=27315; max-age=86400; domain=.baidu.com; path=/','BDORZ2=27315; max-age=86400; domain=.baidu.com; path=/']
    cookies = {}
    for cookie in l:
        k_v = cookie.split(";")[0]
        k_v = k_v.split("=")
        cookies[k_v[0]] = k_v[1]
    return cookies


def getCookieFromStr(cookie):
    lines = cookie.split(";")
    cookies = {}
    for line in lines:
        name, value = line.strip().split('=', 1)
        cookies[name] = value
    return cookies


def getHeaderFromLineStr(headers):
    return {each.split(':', 1)[0].strip(): each.split(':', 1)[1].strip() for each in headers.split('\n') if each.split()}


# 获取字符串中的数字
def getNumFromStr(msg):
    if isinstance(msg, str):
        return re.findall("\d+", str(msg))
    elif isinstance(msg, bytes):
        return re.findall("\d+", str(msg.encode("utf-8")))
    else:
        return msg


#  获取文本中的电话
def getTel(text):
    tel = re.findall(r"\(?0\d{2,3}[) -]?\d{7,9}-\d{0,8}", text)
    if tel:
        return tel[0]
    else:
        tel = re.findall(r"1\d{10}$", text)
        if tel:
            return tel[0]
        return


def getCSSHTML1():
    font_path = "e:\\1.ttf"
    try:
        font = ttLib.TTFont(font_path)
        font.saveXML("E:\\1.xml")
        for one in font.getGlyphOrder():
            print(one)
        return
        num = -1
        for one in font.getGlyphNames():
            print(one)
            continue
            num += 1
            _from = "xe" + one[4:].lower() + ";"
            _to = words[num]
            content = str(content).replace(_from, _to)
        return content
    except:
        print("can't wire font file")
        traceback.print_exc()
        return ""


def modify_data():
    data = "<span style='font-family: myfont;'>&#xedfa;</span>弟目前年入35万左右"
    font_path = "e:\\1.ttf"
    font = ttLib.TTFont(font_path)
    uniList = font['cmap'].tables[0].ttFont.getGlyphOrder()
    print(uniList)

    # 返回替换后的字符串
    return data


def tff2Unicode():  # 将字体映射为unicode列表
    filename = '/home/jason/workspace/1.ttf'
    fnt = fontforge.open(filename)
    for i in fnt.glyphs():
        print(i.unicode)


if __name__ == '__main__':
    pass



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/9/22 0:57
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : Tax.py
# @Software : PyCharm
from mongoengine import *


class Tax(Document):
    email = StringField(required=True)
    first_name = StringField(max_length=50)
    last_name = StringField(max_length=50)


class Comment(EmbeddedDocument):
    content = StringField()
    name = StringField(max_length=120)


class User(Document):
    email = StringField(required=True)
    first_name = StringField(max_length=50)
    last_name = StringField(max_length=50)


class Post(Document):
    title = StringField(max_length=120, required=True)
    author = ReferenceField(User, reverse_delete_rule=CASCADE)
    tags = ListField(StringField(max_length=30))
    comments = ListField(EmbeddedDocumentField(Comment))
    meta = {'allow_inheritance': True}


class TextPost(Post):
    content = StringField()


class ImagePost(Post):
    image_path = StringField()


class LinkPost(Post):
    link_url = StringField()


if __name__ == '__main__':
    ross = User(email='ross@example.com', first_name='Ross', last_name='Lawley').save()
    pass



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time        : 2018/7/7 2:01
# @Email       : billsteve@126.com
# @File          : GlobalHelper.py
# @Software  : PyCharm
import time
import XX.DB.RedisHelper as credis
import XX.String.StringHelper as cstr


# 全局暂停功能
def global_sleep(ts=0, redis_conn=None, **kw):
    if not redis_conn:
        redis_conn = credis.RedisHelper.get_redis_connect(host=kw.get("redis_host", "localhost"))
    if not ts:
        ts = cstr.str2int(redis_conn.get("k_global_ts"))
        ts = ts if ts else 0
    time.sleep(ts)



# -*- coding: utf-8 -*-
import hashlib


class Encrypt():
    @staticmethod
    def md5(string):
        return hashlib.new("md5", string.encode("utf-8")).hexdigest()


if __name__ == '__main__':
    print(Encrypt.md5("1"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/11/27 13:51
# @Author   : Bill
# @Email      : billsteve@126.com
# @Des       : 
# @File        : MongoConn
# @Software: PyCharm


def get_mongo_conn_cfg(**kw):
    d = dict()
    d["host"] = kw.get("host", "localhost")
    # d["username"] = kw.get("username", None)
    # d["password"] = kw.get("pwd", None)
    d["port"] = kw.get("port", 27017)
    return d


ubuntu_cfg = get_mongo_conn_cfg(host="192.168.1.44")
zhihan00_cfg = get_mongo_conn_cfg(host="192.168.1.50")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import base64
import random
from logzero import logger

import XX.DB.RedisHelper as RedisHelper


class IPPollingProxy(object):
    # ping from all provices, then get cdn ip
    proxys_cdn = []

    # crawl everyday, then check them
    proxys_public = []

    # You must be allowed within the range of IP
    proxys_private = [
        'proxy.scrapy.com:23088',
    ]

    # all proxy pool
    proxys = proxys_public + proxys_private

    def process_request(self, request, spider):
        proxy_ip = random.choice(self.proxys)
        if proxy_ip:
            request.meta['proxy'] = "http://" + proxy_ip
        # 验证代理用户名和密码
        # proxy_user_pass = "USERNAME:PASSWORD"
        # setup basic authentication for the proxy
        # encoded_user_pass = base64.encodestring(proxy_user_pass)
        # request.headers['Proxy-Authorization'] = 'Basic ' + encoded_user_pass
        # 根据国内国外ip确定启动哪一组代理池
        # ip发送到ping.chinaz.com查询地址
        # 根据地址来判断启动代理池
        # if u'国内' in area:
        #     request.meta['proxy'] = '这里设置国内http代理'
        # elif u'国外' in area:
        #     request.meta['proxy'] = '这里设置国外http代理'
        # else:
        #     pass
        return


class AbuyunProxy(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def process_request(self, request, spider):
        # 代理服务器
        proxyServer = "http://http-dyn.abuyun.com:9020"
        proxyAuth = "Basic " + base64.urlsafe_b64encode(bytes((AbuyunProxy.settings.get("PROXY_UN") + ":" + AbuyunProxy.settings.get("PROXY_PWD")), "ascii")).decode("utf8")
        request.meta["proxy"] = proxyServer
        request.headers["Proxy-Authorization"] = proxyAuth


class IpProxy(object):

    def __init__(self):
        # 获取Redis连接
        pass

    def process_request(self, request, spider):
        #  这是ip用例，如果想要实时更新，就可以从DB中获取
        ips = ["49.85.6.229:39896", "49.85.6.229:39896", "49.85.6.229:39896"]
        ip = random.choice(ips)
        request.meta["proxy"] = ip


# Goubanjia代理
class Goubanjia(object):

    def __init__(self):
        # 获取Redis连接
        pass

    def process_request(self, request, spider):
        #  这是ip用例，如果想要实时更新，就可以从DB中获取
        ips = ["49.85.6.229:39896"]
        ip = random.choice(ips)
        request.meta["proxy"] = ip


class MimvpProxy(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def __init__(self):
        self.conn_redis = RedisHelper.RedisHelper.get_redis_connect(self.settings.get("REDIS_HOST"), pwd=self.settings.get("REDIS_PWD"), db=9)

    def process_request(self, request, spider):
        ips = self.conn_redis.keys()
        if ips:
            request.meta["proxy"] = "http://c1259e45c76b:b7b9bd6b48@" + random.choice(ips)
            logger.info("Proxy is " + str(request.meta["proxy"]))
        else:
            logger.info("No Proxy ip")



# -*- coding:utf-8 -*-
import XX.File.FileHelper as cf


def log_file(file_path, content, method="a"):
    try:
        return cf.FileHelper.saveFile(file_path, content, method)
    except:
        import traceback
        traceback.print_exc()
        return


def logToady(file_path, content):
    pass


def get_logger():
    import logging
    logging.basicConfig(level=logging.DEBUG, format='%(asctime)s - %(filename)s[line:%(lineno)d] - %(levelname)s: %(message)s')  # logging.basicConfig函数对日志的输出格式及方式做相关配置
    # 由于日志基本配置中级别设置为DEBUG，所以一下打印信息将会全部显示在控制台上
    logging.info('this is a loggging info message')
    logging.debug('this is a loggging debug message')
    logging.warning('this is loggging a warning message')
    logging.error('this is an loggging error message')
    logging.critical('this is a loggging critical message')

# get_logger()


#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time        : 2018/7/9 13:01
# @Email       : billsteve@126.com
# @File          : UrlInfoHelper.py
# @Software  : PyCharm
import json

import XX.File.FileHelper as cf


class UrlInfoHelper(object):

    # json str 2 redis。把异常的url记录下来,并把缓存删了
    @staticmethod
    def add2redis(url_info, redis_key, redis_conn, del_cache=0, spider="", getUrlCachePath=None, uniqueUrl=None):
        try:
            redis_conn.rpush(redis_key, json.dumps(url_info))
        except:
            print("==UrlInfoHelper->Add redis is wrong")
        if del_cache:
            cache_fp = getUrlCachePath(uniqueUrl(url_info.get("url")), spider=spider)
            if cf.FileHelper.isFileExit(cache_fp):
                print("Remove cache file + " + cache_fp + "\t url  is " + url_info.get("url"))
                cf.FileHelper.removeFile(cache_fp)
            else:
                print("=====Cache is not exists========" + cache_fp)
        else:
            print("--" * 50, flush=True)


if __name__ == '__main__':
    pass



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2019/5/13 13:56
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : DateEncode.py
# @Des         : 
# @Software : PyCharm
import datetime
import json
from datetime import date


class DateEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        else:
            return json.JSONEncoder.default(self, obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/12/3 13:24
# @Email     : billsteve@126.com
# @Des       : 添加入口URL
# @File        : InitUrl
# @Software: PyCharm
import importlib
import json
import time

import XX.DB.RedisHelper as ur
import XX.DB.SqlAlchemyHelper as sa
import XX.Tools.BloomFilter as BloomFilter
import XX.Tools.BuiltinFunctions as BF
from XX.DB.RedisHelper import *
from logzero import logger


# 添加入库url
class AddInitUrl:

    def __init__(self):
        self.spider = None
        self.url = None
        self.conn_redis = None

    def add(self, **kwargs):
        return self.conn_redis.lpush(self.spider + kwargs.get("suffix", ":start_urls"), self.url)

    # url2redis
    @staticmethod
    def add_url2redis(spider, url, conn_redis, **kwargs):
        return conn_redis.lpush(spider + kwargs.get("suffix", ":start_urls"), url)

    # URL到rcfg
    @staticmethod
    def add_url2rediscfg(spider, url, cfg, **kwargs):
        conn_redis = ur.RedisHelper.get_redis_connect_by_cfg(cfg)
        return conn_redis.lpush(spider + kwargs.get("suffix", ":start_urls"), url)

    # 把数据库中的某一列添加到某个队列中，并且值作为url_fun的参数，添加的是返回值
    @staticmethod
    def add_table_column2redis(pro_num, *args, column=None, url_fun=None, process_num=10, fn="", spider="", module_name="",
                               class_name=None, r_cfg=None, m_cfg=None, service=True, from_id=None, limit=3000, **kwargs):
        session = sa.SqlAlchemyHelper.get_session_by_cfg(m_cfg)
        conn_redis = ur.RedisHelper.get_redis_connect_by_cfg(r_cfg)
        if kwargs.get("del_q"):
            conn_redis.delete(spider + kwargs.get("suffix", ":start_urls"))
        if from_id is None:
            from_id = conn_redis.get("kid_" + str(fn) + "_" + class_name + "_" + str(pro_num) + "_from_id")
            from_id = from_id if from_id else 0
            logger.info("From id is \t" + str(from_id))

        while 1:
            if conn_redis.llen(spider + kwargs.get("suffix", ":start_urls")) > limit:
                BF.print_from_head("===Too much\t" + class_name + "\t")
                time.sleep(2 * (pro_num + 1))
                continue
            model_class = getattr(importlib.import_module(module_name), class_name)
            infos = model_class.getByFromIdAndMod(from_id, process_num, pro_num, session, limit=10)
            if infos:
                for info in infos:
                    if url_fun:
                        url = url_fun(info.__dict__.get(column))
                    else:
                        url = info.__dict__.get(column)
                    if url:
                        url = url.strip()
                        if kwargs.get("bf"):
                            bloomFilter = BloomFilter.BloomFilter(conn_redis, key=spider)
                            if bloomFilter.is_exists(url):
                                BF.print_no_end("-")
                            else:
                                res = conn_redis.lpush(spider + kwargs.get("suffix", ":start_urls"), url)
                                logger.info(str((spider, res, info.id, url)))
                                bloomFilter.add(url)
                        else:
                            res = conn_redis.lpush(spider + kwargs.get("suffix", ":start_urls"), url)
                            logger.info(str((spider, res, info.id, url)))

                    from_id = info.id
                    conn_redis.set("kid_" + str(fn) + "_" + class_name + "_" + str(pro_num) + "_from_id", from_id)
            else:
                if service:
                    BF.print_from_head("No More\t" + class_name + "\t")
                    time.sleep(2 * (pro_num + 1))
                    session.commit()
                else:
                    return

    # 文件到队列
    @staticmethod
    def add_file2redis(spider, fp, conn_redis, line_func=None):
        for url in open(fp, encoding="utf-8"):
            if line_func:
                url = line_func(url)
            AddInitUrl.add_url2redis(spider, url, conn_redis)

    # 添加check url
    @staticmethod
    def addCheckUrl(project, url, spider, conn, suffix=":start_urls:check"):
        d = dict()
        d["url"] = url
        d["project"] = project
        d["spider"] = spider
        url = json.dumps(d, ensure_ascii=False)
        return conn.lpush(spider + suffix, url)

    # 把check的URL添加到IU中
    @staticmethod
    def addInitUrlFromCheck(hcfg, rcfg, getRow, ts=0):
        import XX.DB.HappyBaseHelper as HaB

        conn_redis = RedisHelper.get_redis_connect_by_cfg(rcfg)
        conn_hbase = HaB.HappyBaseHeleper.get_connection_by_cfg(hcfg)
        # pool = HaB.HappyBaseHeleper.getPoolByCfg(hcfg)
        while 1:
            keys = conn_redis.keys("*:start_urls:check")
            if not keys:
                BF.print_from_head("No More Check IU in " + str(rcfg["host"]), ts=ts)
                continue
            for key in keys:
                jd = json.loads(conn_redis.lpop(key))
                url = jd["url"]
                if url:
                    # table = HaB.HappyBaseHeleper.getTable("crawl_" + jd["project"], pool=pool)
                    table = HaB.HappyBaseHeleper.get_table("crawl_" + jd["project"], conn=conn_hbase)
                    # HBase是否存在
                    row = getRow(url=url)
                    if row:
                        exists = HaB.HappyBaseHeleper.get_row(row)
                        if not exists:
                            res = conn_redis.lpush(key[:-6], url)
                            print("Add new IU res \t\t" + str(res))
                        else:
                            print("Already Crawled!\t\t" + url)
                    else:
                        print("==== No row key", jd)
                time.sleep(ts)

    # 重新添加不是200的url到Check
    @staticmethod
    def readdNot200(rcfg, ts=10, suffix=":init_urls"):
        conn_redis = RedisHelper.get_redis_connect_by_cfg(rcfg)
        while 1:
            keys = conn_redis.keys("*not200*")
            if not keys:
                BF.print_from_head("No More Spider in " + str(rcfg["host"]), ts=ts)
                continue
            for key in keys:
                url = conn_redis.spop(key)
                if url:
                    if conn_redis.sadd("s_not_200_urls", url):
                        logger.info("Readd url res is\t" + str(conn_redis.lpush(key.split(":")[0] + suffix, url)) + "\tkey is\t" + key[:-7] + "\t url is \t" + url)
                    else:
                        print("Retry already!")
                else:
                    logger.info("No url in set \t" + str(key))
                time.sleep(ts)



#!/usr/bin/python3
# -*- coding:utf-8 -*-
# @Time       : 2018/9/22 13:25
# @Author    : Bill Steve
# @Email      : billsteve@126.com
# @File         : Tax.py
# @Software : PyCharm
from mongoengine import *


class Tax(Document):
    success = BooleanField(required=True)
    data = DictField(max_length=50)
    url = StringField(max_length=250)
    meta = {'collection': 'tax'}


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/11/27 13:50
# @Author   : Bill
# @Email      : billsteve@126.com
# @Des       : Redis的配置参数
# @File        : Redis
# @Software: PyCharm
import functools


def get_redis_conn_cfg(**kw):
    d = dict()
    d["host"] = kw.get("host", "localhost")
    d["pwd"] = kw.get("pwd", None)
    d["port"] = kw.get("port", 6379)
    d["db"] = kw.get("db", 0)
    d["decode_responses"] = kw.get("decode_responses", True)
    return d


ali_cfg = functools.partial(get_redis_conn_cfg, host="39.104.97.69", pwd="DRsXT5ZJ6Oi55LPQ", db=0)
ali2_cfg = functools.partial(get_redis_conn_cfg, host="39.104.57.130", pwd="DRsXT5ZJ6Oi55LPQ", db=0)
ubuntu_cfg = functools.partial(get_redis_conn_cfg, host="192.168.1.44", db=0)
local = functools.partial(get_redis_conn_cfg, host="localhost", db=0)
zhihan00 = functools.partial(get_redis_conn_cfg, host="zhihan00", db=0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import pickle
import time
import traceback

import happybase
from logzero import logger
from scrapy.http import TextResponse

import XX.Encrypt.EncryptHelper as enc
import XX.File.FileHelper as Fh
from XX.Log.LogHelper import *


# 文件缓存中间件
class CacheFileRequest(object):

    # TODO:test
    @staticmethod
    def get_cache_file(cls, url, spider=None, spider_name=None, root_path_cache="", url_process=None,
                       url2spider=lambda s: s):
        if url_process and callable(url_process):
            url = url_process(url)
        spider = spider_name if spider_name else (spider if spider else url2spider(url).split(":")[0])
        root_path_cache = root_path_cache if root_path_cache else ''
        return root_path_cache + spider + os.sep + Fh.FileHelper.get_md5_name(url) + ".cache"

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.cache_file_path_func = settings.get("FUN_CACHE_FILE_PATH", cls.get_cache_file)
        cls.cache_file_exclude = settings.get("CACHE_FILE_EXCLUDE")
        cls.cache_file_ts = settings.get("CACHE_FILE_TS", None)
        cls.cache_file_exclude_ts = settings.get("CACHE_FILE_EXCLUDE_TS", cls.cache_file_ts)
        return cls()

    def process_request(self, request, spider):
        cache_file_path = self.cache_file_path_func(url=str(request.url), spider=str(spider.name))

        # 是否应该读缓存
        def should_read_cache(request, spider) -> bool:
            # 存在缓存文件
            exists_file = Fh.FileHelper.is_file_exit(cache_file_path)
            # 是排除URL
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts >= 0:

                        if time.time() - Fh.FileHelper.get_create_ts(cache_file_path) < self.cache_file_exclude_ts:
                            return True
                        else:
                            logger.debug(
                                f"缓存文件过期。不读取。{cache_file_path}{Fh.FileHelper.get_create_ts(cache_file_path)}  {request.url}")
                            return False
                    # 排除URL无过期时间,就是排除URL都不读缓存
                    else:
                        logger.debug(f"排除的，没设置过期时间，全不读缓存。{request.url}")
                        return False
                else:
                    # 全部时间
                    if self.cache_file_ts:
                        # 全部URL 没有过期
                        if time.time() - Fh.FileHelper.get_create_ts(cache_file_path) < self.cache_file_ts:
                            return True
                        else:
                            logger.debug(f"不是排除请求，但是缓存文件过期，不读缓存。{request.url}")
                            return False
                    else:
                        return True
            else:
                logger.debug(f"缓存文件不存在。 {request.url}")
                return False

        if should_read_cache(request, spider):
            try:
                logger.info(f"===Read cache===  {cache_file_path.split(os.sep)[-1]}   {request.url}")
                return pickle.load(open(cache_file_path, "rb"))
            except Exception as e:
                logger.info(f"== Can't Read cache ===  Reason is {e}")
                traceback.print_exc()
        else:
            # 不读缓存
            # logger.info("===Should Not Read Cache===   " + request.url)
            pass

    # 写文件缓存
    def process_response(self, request, response, spider):
        cache_file_path = self.cache_file_path_func(url=str(request.url), spider=str(spider.name))

        # 是否应该重写缓存
        def should_write_cache(request, spider) -> bool:
            exists_file = Fh.FileHelper.is_file_exit(cache_file_path)
            exclude = 1 if self.cache_file_exclude and request.url.find(self.cache_file_exclude) >= 0 else 0
            if exists_file:
                if exclude:
                    if self.cache_file_exclude_ts:
                        if time.time() - Fh.FileHelper.get_create_ts(cache_file_path) > self.cache_file_exclude_ts:
                            logger.debug(
                                f"排除缓存文件过期。需要重写。{cache_file_path}{Fh.FileHelper.get_create_ts(cache_file_path)}  {request.url}")
                            return True
                        else:
                            logger.debug(
                                f"排除的URL没过期，不重写。过了{time.time() - Fh.FileHelper.get_create_ts(cache_file_path)} 秒？ {cache_file_path}")
                            return False
                    else:
                        logger.debug(f"排除URL没设置过期时间 都要重写。 {cache_file_path}")
                        return True
                else:
                    # logger.debug(f"不是排除的URL不重写缓存。 {cache_file_path}")
                    return False
            else:
                logger.debug(f"缓存文件不存在。写缓存。 {request.url}")
                return True

        if response.status < 300:
            cf.FileHelper.mkdir(cf.FileHelper.get_file_path_and_name(cache_file_path)[0])
            try:
                if should_write_cache(request, spider):
                    pickle.dump(response, open(cache_file_path, "wb"))
                    logger.info(f"===Write cache===      {cache_file_path}   {request.url}")
                # else:
                #     logger.info(f"===Cache Exists===      {cache_file_path}   {request.url}")
            except Exception as e:
                logger.info(f"== Can't Write cache 2 file==   Reason is {e}")
                traceback.print_exc()
        else:
            logger.info(f"===Status Code ERROR ===   Code is {response.status}" + response.url)
        return response


# 文件按天缓存中间件
class CacheFileByDayRequest(object):

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.cache_file_path_by_day = settings.get("FUN_CACHE_FILE_DAY_PATH")
        return cls()

    def get_cache_response(self, request, spider):
        cache_file_path = CacheFileByDayRequest.cache_file_path_by_day(request.url, spider=spider.name)
        if cf.FileHelper.is_file_exit(cache_file_path):
            try:
                logger.info("===Read cache===\t" + request.url + "\t" + cache_file_path)
                return pickle.load(open(cache_file_path, "rb"))
            except Exception as e:
                logger.info(f"== Can't Read cache ===  Reason is {e}")
                traceback.print_exc()
        return None

    def process_request(self, request, spider):
        response = self.get_cache_response(request, spider)
        logger.info("R>>>>\trow_key\t" + request.url)
        logger.info("===Read cache===\t" + request.url)
        return response if response else None

    # 写文件缓存
    def process_response(self, request, response, spider):
        if response.status == 200:
            cache_response = self.get_cache_response(request, spider)
            if not cache_response:
                # 不存在缓存文件就要写入
                cache_file_path = CacheFileByDayRequest.cache_file_path_by_day(request.url, spider=spider.name)
                if not cf.FileHelper.is_file_exit(cache_file_path):
                    cf.FileHelper.mkdir(cf.FileHelper.get_file_path_and_name(cache_file_path)[0])
                    try:
                        pickle.dump(response, open(cache_file_path, "wb"))
                        logger.info("===Save cache===\t" + cache_file_path)
                    except Exception as e:
                        logger.info(f"== Can't Write cache 2 file==  Reason is  {e}")
                        traceback.print_exc()
        return response


# Redis Hash缓存中间件
class CacheRedisHashRequest(object):

    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        import XX.DB.RedisHelper as RH
        self.conn_redis7 = RH.RedisHelper.get_redis_connect(self.settings.get("REDIS_HOST"),
                                                            port=self.settings.get("REDIS_PORT"),
                                                            pwd=self.settings.get("REDIS_PWD"), db=7)

    # 读redis缓存
    def process_request(self, request, spider):
        if self.conn_redis7.exists(request.url):
            print("==Read cache 2 redis==")
            url = self.conn_redis7.hget(request.url, "url")
            html = self.conn_redis7.hget(request.url, "html")
            status_code = self.conn_redis7.hget(request.url, "status_code")
            encoding = self.conn_redis7.hget(request.url, "encoding")
            return TextResponse(url=url, body=html, status=status_code, encoding=encoding)

    # 写redis缓存
    def process_response(self, request, response, spider):
        if response.status == 200:
            try:
                if not self.conn_redis7.exists(response.url):
                    self.conn_redis7.hset(response.url, "url", response.url)
                    self.conn_redis7.hset(response.url, "html", response.text)
                    self.conn_redis7.hset(response.url, "status_code", response.status)
                    self.conn_redis7.hset(response.url, "encoding", response.encoding)
                    self.conn_redis7.hset(response.url, "spider", spider.name)
                    self.conn_redis7.hset(response.url, "project_name", self.settings.get("PROJECT_NAME", "crawl"))
                    print("==Write cache 2 redis==")
            except:
                print("==Can't Write cache 2 redis==")
                traceback.print_exc()
        return response


# Redis缓存中间件
class CacheRedisRequest(object):
    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        import XX.DB.RedisHelper as RH
        self.conn_redis14 = RH.RedisHelper.get_redis_instance()

    # 读redis缓存
    def process_request(self, request, spider):
        try:
            content = self.conn_redis14.get(request.url)
            if content:
                try:
                    response = pickle.loads(content)
                except:
                    self.conn_redis14.delete(request.url)
        except:
            print("==Can't Read cache from redis==")
            traceback.print_exc()

    # 写redis缓存
    def process_response(self, request, response, spider):
        if response.status == 200:
            try:
                if not self.conn_redis14.exists(response.url):
                    pickinfo = pickle.dumps(response)
                    self.conn_redis14.set(response.url, pickinfo)
            except:
                print("==Can't Write cache 2 redis==")
                traceback.print_exc()
        return response


# # HBASE缓存下载中间件
# class CacheHBaseRequest(object):
#     @classmethod
#     def from_crawler(cls, crawler):
#         settings = crawler.settings
#         cls.settings = settings
#         return cls()
#
#     def __init__(self):
#         transport = TSocket.TSocket(self.settings.get("HBASE_HOST", "localhost"), self.settings.get("HBASE_PORT", 9090))
#         transport.open()
#         self.protocol = TBinaryProtocol.TBinaryProtocol(transport)
#         self.client = Hbase.Client(self.protocol)
#
#         # 判断是否有表，没有则生成
#         tables = self.client.getTableNames()
#         self.table_name = "crawl_" + self.settings.get("PROJECT_NAME", "crawl")
#         if self.table_name not in tables:
#             source = ColumnDescriptor(name='source')
#             data = ColumnDescriptor(name='data')
#             self.client.createTable(self.table_name, [source, data])
#             print("Create_table" + "===" * 10)
#
#     # 读取缓存
#     def getCacheResult(self, request, spider):
#         columns = ["source:url", "source:html", "source:status_code", "source:size"]
#         result = self.client.getRowWithColumns(self.table_name, spider.name + "_" + enc.Encrypt.md5(request.url), columns)
#         if result:
#             if time.time() - result[0].columns.get('source:html').timestamp // 1000 <= int(self.settings.get("HBASE_EXPIRE_TS", 0)):
#                 return result
#         return None
#
#     # 读缓存，生成response
#     def process_request(self, request, spider):
#         result = self.getCacheResult(request, spider)
#         if result:
#             logger.info("Read response from hbase" + ">>>\t" + request.url)
#             col = result[0].columns
#             encoding = col.get('source:encoding').value if col.get('source:encoding') else "utf-8"
#             return TextResponse(url=col.get('source:url').value, body=col.get('source:html').value, status=col.get('source:status_code').value, encoding=encoding)
#
#     # 写缓存到HBASE
#     def process_response(self, request, response, spider):
#         if not self.getCacheResult(request, spider):
#             if response.status == 200:
#                 row = spider.name + "_" + enc.Encrypt.md5(response.url)
#                 mutations = list()
#                 mutations.append(Mutation(column="source:url", value=str(response.url)))
#                 mutations.append(Mutation(column="source:status_code", value=str(response.status)))
#                 mutations.append(Mutation(column="source:html", value=str(response.text)))
#                 mutations.append(Mutation(column="source:type", value="html"))
#                 mutations.append(Mutation(column="source:size", value=str(len(response.text))))
#                 mutations.append(Mutation(column="source:encoding", value=response.encoding))
#                 self.client.mutateRow(self.table_name, row, mutations)
#                 logger.info("Cache response 2 hbase" + "<<<\t" + response.url)
#             else:
#                 logger.info("Not 200" + "---\t" + response.url)
#         return response


# HBASE缓存下载中间件
class CacheHappyBaseRequest(object):
    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        pool = happybase.ConnectionPool(size=3, host=self.settings.get("HBASE_HOST", "localhost"),
                                        port=self.settings.get("HBASE_PORT", 9090), protocol='binary',
                                        transport='buffered')
        self.connection = pool._acquire_connection()
        self.table_name = "crawl_" + self.settings.get("PROJECT_NAME", "crawl")
        self.table = self.connection.table(self.table_name)

        # 判断是否有表，没有则生成
        tables = self.connection.tables()
        if self.table_name.encode("utf-8") not in tables:
            self.connection.create_table(self.table_name, {'source': {}, 'data': {}})

    # 读取缓存
    def getCacheResult(self, request, spider, row_key=None):
        if not row_key:
            row_key = spider.name + "_" + enc.Encrypt.md5(request.url)
        try:
            row = self.table.row(row_key, include_timestamp=True)
            if row:
                # 默认HTML是永久存储的
                if (request.url.endswith("html") or request.url.endswith("htm")) and not self.settings.get(
                        "HBASE_HTML_EXPIRE", 0):
                    return row
                expire = int(self.settings.get("HBASE_EXPIRE_TS", 0))
                if expire > 0:
                    if time.time() - row.get(b'source:url')[1] // 1000 <= expire:
                        return row
                    else:
                        logger.info(str(row.get(b'source:url')[1]) + "\tis out time\t" + str(expire))
                else:
                    return row
        except:
            self.__init__()
            logger.info("Reconnect 2 HBase" + "===" * 33)
        return None

    # 读缓存，生成response
    def process_request(self, request, spider):
        row_key = spider.name + "_" + enc.Encrypt.md5(request.url)
        result = self.getCacheResult(request, spider, row_key=row_key)
        if result:
            logger.info("R>>>>\trow_key\t" + str(row_key) + "\t" + request.url)
            encoding = result.get(b"source:encoding", [b"utf-8"])[0].decode("utf-8")
            return TextResponse(url=result.get(b"source:url")[0].decode("utf-8"),
                                body=result.get(b"source:html")[0].decode("utf-8"),
                                status=result.get(b"source:status_code")[0].decode("utf-8"), encoding=encoding)
        else:
            pass

    # 写缓存到HBASE
    def process_response(self, request, response, spider):
        if not self.getCacheResult(request, spider):
            if response.status == 200:
                data = {
                    "source:url": str(response.url),
                    "source:status_code": str(response.status),
                    "source:html": str(response.text),
                    "source:type": "html",
                    "source:size": str(len(response.text)),
                    "source:encoding": response.encoding
                }
                row_key = spider.name + "_" + enc.Encrypt.md5(request.url)
                try:
                    self.table.put(row_key, data)
                except:
                    self.__init__()
                    logger.info("Reconnect 2 HBase" + "===" * 33)
                logger.info("W<<<<\trow_key\t" + str(row_key) + "\t" + response.url)
            else:
                logger.info("Not 200 ---\t" + str(response.status) + "\t" + response.url)
        return response

    def close_spider(self):
        pass


# HBaseE缓存下载中间件
class CacheHBaseRK(object):
    @classmethod
    def from_crawler(cls, crawler):
        settings = crawler.settings
        cls.settings = settings
        return cls()

    def __init__(self):
        pool = happybase.ConnectionPool(size=3, host=self.settings.get("HBASE_HOST", "localhost"),
                                        port=self.settings.get("HBASE_PORT", 9090), protocol='binary',
                                        transport='buffered')
        self.connection = pool._acquire_connection()
        self.table_name = "crawl_" + self.settings.get("PROJECT_NAME", "crawl")
        self.table = self.connection.table(self.table_name)

        # 判断是否有表，没有则生成
        tables = self.connection.tables()
        if self.table_name.encode("utf-8") not in tables:
            self.connection.create_table(self.table_name, {'source': {}, 'data': {}})

    # 读取缓存
    def getCacheResult(self, request, spider, row_key=None):
        if not row_key:
            row_key = self.settings.get("HBASE_ROW_KEY_FUN")(request=request)
            print("RRRRRRRRRRKKKKKKKK", row_key)
        try:
            row = self.table.row(row_key, include_timestamp=True)
            if row:
                # 默认HTML是永久存储的
                if (request.url.endswith("html") or request.url.endswith("htm")) and not self.settings.get(
                        "HBASE_HTML_EXPIRE", 0):
                    return row
                expire = int(self.settings.get("HBASE_EXPIRE_TS", 0))
                if expire > 0:
                    if time.time() - row.get(b'source:url')[1] // 1000 <= expire:
                        return row
                    else:
                        logger.info(str(row.get(b'source:url')[1]) + "\tis out time")
                else:
                    return row
        except:
            self.__init__()
            logger.info("Reconnect 2 HBase" + "===" * 33)
        return None

    # 读缓存，生成response
    def process_request(self, request, spider):
        row_key = self.settings.get("HBASE_ROW_KEY_FUN")(request=request)
        result = self.getCacheResult(request, spider, row_key=row_key)
        if result:
            logger.info("R>>>>\trow_key\t" + str(row_key) + "\t" + request.url)
            encoding = result.get(b"source:encoding", [b"utf-8"])[0].decode("utf-8")
            return TextResponse(url=result.get(b"source:url")[0].decode("utf-8"),
                                body=result.get(b"source:html")[0].decode("utf-8"),
                                status=result.get(b"source:status_code")[0].decode("utf-8"), encoding=encoding)

    # 写缓存到HBASE
    def process_response(self, request, response, spider):
        if not self.getCacheResult(request, spider):
            if response.status == 200:
                data = {
                    "source:url": str(response.url),
                    "source:status_code": str(response.status),
                    "source:html": str(response.text),
                    "source:type": "html",
                    "source:size": str(len(response.text)),
                    "source:encoding": response.encoding
                }
                row_key = self.settings.get("HBASE_ROW_KEY_FUN")(request=request)
                try:
                    self.table.put(row_key, data)
                except:
                    self.__init__()
                    logger.info("Reconnect 2 HBase" + "===" * 33)
                logger.info("W<<<<\trow_key\t" + str(row_key) + "\t" + response.url)
            else:
                logger.info("Not 200 --- " + "\t" + response.url)
        return response



# -*- coding: utf-8 -*-
import re


class RegExpHelper():

    # TODO：debug 其他图片 以img开头的图片的url
    @staticmethod
    def get_img_urls(html):
        return re.findall('http.+\.jpg', html)

    @staticmethod
    def get_url_from_html(txt):
        return re.findall(r"(?<=href=\").+?(?=\")|(?<=href=\').+?(?=\')", txt)

    @staticmethod
    def get_url_from_text(txt):
        return re.findall(r"http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+", txt)

    @staticmethod
    def get_ip(txt):
        return re.findall(r"((?:(?:25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d)))\.){3}(?:25[0-5]|2[0-4]\d|((1\d{2})|([1-9]?\d))))", txt)

    @staticmethod
    def replace_continuous_blank_2_one(txt):
        return re.sub(' +', ' ', txt)

    @staticmethod
    def get_date(txt):
        if not txt or isinstance(txt, str):
            return
        return re.findall(r"(\d{4}-\d{1,2}-\d{1,2})", txt)

    @staticmethod
    def repleaseEmoji(sourceStr, replaceStr=''):
        if isinstance(sourceStr, str):
            try:
                co = re.compile('[\U00010000-\U0010ffff]')
            except re.error:
                import traceback
                co = re.compile('[\uD800-\uDBFF][\uDC00-\uDFFF]')
            return co.sub(sourceStr, replaceStr)
        else:
            return sourceStr


if __name__ == "__main__":
    print(RegExpHelper.repleaseEmoji("加油， 我们与你同在！[抱抱]🌹", "??111"))
    exit()
    content = "<td class='info_d'><div class='info'><a href='//car.autohome.com.cn/pic/series-s17694/49.html?pvareaid=101281'>图片</a><a href='spec/17694/config.html?pvareaid=101282'>参数配置</a>"
    content = "https://www.jb51.net/article/98054.htm"
    for l in RegExpHelper.get_url_from_text(content):
        print(l)



# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

setup(
    name="XX",
    version="1.1584539479",
    description=(
        "Python tools for myself(billsteve@126.com)"
    ),
    long_description="""**Just Test, Just for myself!** 
    
    Not safe,not stable and will boom boom boom.

    Just like these:
        -  https://music.163.com/song?id=3949444&userid=319278554  
        -  https://music.163.com/song?id=474581010&userid=319278554  
        -  https://open.spotify.com/album/47wyCwrChF50ZTFNOuWx99  
        -  https://open.spotify.com/track/3oDFtOhcN08qeDPAK6MEQG  
    """,
    author='bill steve',
    author_email='billsteve@126.com',
    maintainer='bill steve',
    maintainer_email='billsteve@126.com',
    license='MIT License',
    packages=find_packages(),
    platforms=["all"],
    url='https://github.com/billsteve',

    install_requires=[
        "scrapy",
        "scrapyd",
        "scrapyd-client",
        "scrapy-redis",
        "pyquery",
        "redis",
        "requests",
        "pymysql",
        "sqlalchemy",
        "logzero",
        "happybase",
        "tld"
    ],

    classifiers=[
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time     : 2018/12/17 10:53
# @Email     : billsteve@126.com
# @Des       : 
# @File        : LogDM
# @Software: PyCharm
# 记录开始抓了哪些url

import XX.Date.DatetimeHelper as ctime


class ToCrawlUrl(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def process_request(self, request, spider):
        save_path = ToCrawlUrl.settings.get("ROOT_PATH_LOG") + ctime.get_today() + "_to_crawl.txt"
        log_file(save_path, str(ctime.get_now_time()) + "\t" + spider.name + "\t" + request.url + "\n", method="a")


# 记录成功抓了哪些url
class CrawledUrl(object):

    @classmethod
    def from_crawler(cls, crawler):
        cls.settings = crawler.settings
        return cls()

    def process_response(self, request, response, spider):
        save_path = CrawledUrl.settings.get("ROOT_PATH_LOG") + ctime.get_today() + "_carwled.txt"
        log_file(save_path, str(ctime.get_now_time()) + "\t" + spider.name + "\t" + str(response.status) + "\t" + request.url + "\n",
                method="a")
        return response





from mongoengine import *


class TaxInfo(Document):
    web_key = StringField(required=True, max_length=50)
    tax = IntField()

    @staticmethod
    def getByWebKey(web_key):
        return TaxInfo.objects(web_key=web_key)

    @staticmethod
    def saveToMongo(item):
        data = TaxInfo(web_key=item.web_key, tax=1)
        res = data.save()
        print(res)



#!/usr/bin/python3
# -*- coding:utf-8 -*-
import hashlib
import os

from logzero import logger
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker


def get_session(host="localhost", user="root", password="123456", port="3306", charset="utf-8", db="test", *arg, **kw):
    global session
    if session:
        return session
    else:
        driver = kw.get("driver", "pymysql")
        mysql_url = 'mysql+' + driver + '://' + user + ":" + password + "@" + host + ":" + str(port) + "/" + str(
            db) + "?charset=" + charset.replace("-", "") + "&autocommit=true"
        engine = create_engine(mysql_url, encoding="utf8", *arg, **kw)
        session = sessionmaker(bind=engine)()
        return session


def get_md5_name(hash_string):
    md5_str = hashlib.new("md5", hash_string.encode("utf-8")).hexdigest()
    return md5_str[0] + os.sep + md5_str[1] + os.sep + md5_str[2] + os.sep + md5_str


def writefile(filename, text, mode="w"):
    f = open(filename, mode, encoding="UTF-8")
    f.write(text)
    f.close()
    pass


####################################################
import pymysql

global_conn = None


def connect(host="localhost", user="root", password="123456", port="3306", charset="utf8", db="test",
            mysql_timeout=5000, use_global=False):
    global global_conn
    if global_conn:
        logger.info("使用全局连接")
        return global_conn

    try_times = 3
    conn = None
    for i in range(try_times):
        try:
            conn = pymysql.connect(host=host, user=user, password=password, db=db, port=port, charset=charset,
                                   connect_timeout=mysql_timeout, autocommit=1)
            break
        except Exception as e:
            import traceback
            traceback.print_exc()
            logger.info("data connection error,%s" % e)
        finally:
            pass
    if use_global:
        global_conn = conn
        logger.info("首次连接")
    return conn


def close(conn):
    global global_conn
    if global_conn:
        return False
    else:
        pass
    try:
        conn.close()
    except Exception as e:
        logger.info("data shutdown error,%s" % e)
    finally:
        pass

    return True


def insert(conn, sql):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor()
        cur.execute(sql)
        id = int(conn.insert_id())
        conn.commit()
    except Exception as e:
        logger.info("Mysql Insert Error: %s, %s" % (e, sql))
        logger.info("data insert error")
        conn.rollback()
        id = -1
    finally:
        cur.close()
    return id


def lastrowid(conn):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor()
        id = int(cur.lastrowid)
    except Exception as e:
        logger.info("data last_id error,%s" % e)
        id = -1
    finally:
        cur.close()
    return id


def update(conn, sql):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor(pymysql.cursors.DictCursor)
        result = cur.execute(sql)
        conn.commit()
    except Exception as e:
        logger.info("Mysql Update Error: %s" % e)
        writefile("mysql-err.txt", sql + "\n", mode="a")
        logger.info("data update error")
        conn.ping()
        conn.rollback()
        result = False
    finally:
        cur.close()
    return result


def fetch(conn, sql):
    global global_conn
    conn = global_conn if global_conn else conn
    try:
        conn.ping()
        cur = conn.cursor(pymysql.cursors.DictCursor)
        rs = cur.execute(sql)
        list = cur.fetchmany(rs)
        conn.commit()
    except Exception as e:
        logger.info("Mysql Fetch Error: %s" % e)
        logger.info("data fetch error")
        conn.rollback()
        conn.ping()
        list = None
    finally:
        cur.close()
    return list


def make_insert_sql(table, data, updates, replace=1):
    columnssql = ""
    valuessql = ""
    updatessql = ""
    for key in data.keys():
        if data[key]:
            data[key] = pymysql.escape_string(str(data[key]))
        columnssql = columnssql + "`" + key + "`,\t"
        if data[key] is not None:
            valuessql = valuessql + "'" + str(data[key]) + "',\t"
        else:
            valuessql = valuessql + "null,\t"

        if replace:
            # 覆盖
            if key in updates:
                if data[key] is not None:
                    updatessql = updatessql + "`" + key + "` = '" + str(data[key]) + "',\t"
                else:
                    updatessql = updatessql + "`" + key + "` = null,\t"
        else:
            # 补充
            if key in updates:
                if data[key]:
                    updatessql = updatessql + "`" + key + f"` =  CASE WHEN {key}  IS NOT NULL THEN {key} ELSE '" + str(
                        data[key]) + "'  END,\t"

    sql = "INSERT INTO `#tablename#`( #columns# )VALUES( #values# ) ON DUPLICATE KEY UPDATE #updates# ;\t"
    columnssql = columnssql[:-2]
    valuessql = valuessql[:-2]
    if len(updatessql) > 2:
        updatessql = updatessql[:-2]
    sql = sql.replace("#tablename#", table)
    sql = sql.replace("#columns#", columnssql)
    sql = sql.replace("#values#", valuessql)
    sql = sql.replace("#updates#", updatessql)
    return sql


if __name__ == "__main__":
    data = {
        "a": 1,
        "b": None,
        "c": "hha"
    }
    sql = make_insert_sql("t", data, data, 0)
    print(sql)
